#ifndef _AST_
#define _AST_
#include <fstream>
#include <iostream>
#include <string>
#include <vector>
using namespace std;

//-- node-type codes: stmt codes are even, exp codes are odd
const int _asn = 2, _if = 4, _while = 6, _read = 8, _write = 10, _new = 12;
const int _binop = 1, _unop = 3, _num = 5, _bool = 7, _lambda = 9, _funcall = 11, _id = 13;

//-- add endscope node: endscope:stmt -> ID stmt
const int _endscope = 14;
const int _expstmt = 16;
const int _builtin = 15;

//-- op-codes: i.e. what kind of _unop or _binop do we have?
const int _lt = 1, _eq = 2, _le = 3, _gt = 4, _ne = 5, _ge = 6,
  _sum = 8, _dif = 9, _mul = 10, _div = 11, _uplus = 12, _uminus = 13,
  _and = 14, _or = 15, _not = 16; 

//-- Node: abstract syntax tree node
class Node
{
 public:  
  int ntype;     //-- holds the node-type code, i.e. what kind of stmt or exp
  int ibVal;     //-- value for _num/_bool nodes, op-code for _binop/_unop nodes
  string idVal;  //-- string value of _id nodes
  vector<Node*> child;

  Node(int a1, int a2, const string& a4="", 
       Node *a5=0, Node *a6=0, Node *a7 = 0, Node *a8 = 0, Node *a9 = 0);
  Node(const Node& N): ntype(N.ntype),ibVal(N.ibVal),idVal(N.idVal),child(N.child){}
};

//-- Adds stmt-node nxt to back of stmt-list rooted at stmt-node L
void addToBack(Node* L, Node *nxt);

//-- Displays (using ggv) the abstract syntax tree rooted at "root"
void displayAST(Node *root);

//-- Adds _endscope nodes to AST for each variable created
void addCloseScopeStmts(Node* T);

#endif
