%{
#define YYSTYPE Node*

#include "ast.h"
#include "lab14.tab.hpp"

%}

%option noyywrap

%%

[0-9]+     { yylval = new Node(_num,atoi(yytext)); return NUM; }
true|false { yylval = new Node(_bool, yytext[0] == 't'); return BCONST; }
"+"|"-"    { yylval = new Node(_binop, yytext[0] == '+' ? _sum : _dif); return OPA; }
"*"|"/"    { yylval = new Node(_binop, yytext[0] == '*' ? _mul : _div); return OPM; }
"("        { yylval = 0; return LP; }
")"        { yylval = 0; return RP; }
"or"       { yylval = new Node(_binop,_or); return OROP; }
"and"      { yylval = new Node(_binop,_and); return ANDOP; }
"!"        { yylval = new Node(_unop,_not); return NOT; }
"<"        { yylval = new Node(_binop,_lt); return COMPOP; }
">"        { yylval = new Node(_binop,_gt); return COMPOP; }
"<="       { yylval = new Node(_binop,_le); return COMPOP; }
">="       { yylval = new Node(_binop,_ge); return COMPOP; }
"="        { yylval = new Node(_binop,_eq); return COMPOP; }
"!="       { yylval = new Node(_binop,_ne); return COMPOP; }
":="       { yylval = 0; return ASN; }
"if"       { yylval = 0; return IF; }
"else"     { yylval = 0; return ELSE; }
"while"    { yylval = 0; return WHILE; }
"read"     { yylval = 0; return READ; }
"write"    { yylval = 0; return WRITE; }
"lambda"   { yylval = 0; return LAMBDA; }
"new"      { yylval = 0; return NEW; }
"{"        { yylval = 0; return LB; }
"}"        { yylval = 0; return RB; }
";"        { yylval = 0; return STMTTERM; }
[a-zA-Z][0-9a-zA-Z]* { yylval = new Node(_id,0,yytext); return ID; }
<<EOF>> { return 0; }
[ \n\t] { }
#.*\n   { }
%%
